from django.db.models.signals import post_save, pre_save, post_delete
from django.dispatch import receiver
from django_rq import get_queue
from dcim.models import Interface, Device, DeviceRole, DeviceType, Site
from ipam.models import Prefix
from netbox.config import get_config
from netbox.constants import RQ_QUEUE_DEFAULT
from .models import AWXInventory

import logging


logger = logging.getLogger(__name__)

__all__ = [
    "update_host",
]


queue_name = get_config().QUEUE_MAPPINGS.get("awx", RQ_QUEUE_DEFAULT)


@receiver(post_save, sender=Site)
@receiver(post_save, sender=DeviceRole)
@receiver(post_save, sender=DeviceType)
@receiver(post_save, sender=Prefix)
def save_group_instance(sender, instance, created, **kwargs):
    rq_queue = get_queue(queue_name)
    for inventory in AWXInventory.objects.all():
        if inventory.enabled == True:
            params = {"inventory": inventory, "sender": sender, "instance": instance}
            rq_queue.enqueue("netbox_awx_plugin.synchronization.sync_group", **params)


@receiver(pre_save, sender=Site)
@receiver(pre_save, sender=DeviceRole)
@receiver(pre_save, sender=DeviceType)
@receiver(pre_save, sender=Prefix)
def pre_save_group_instance(sender, instance, **kwargs):
    if sender.objects.filter(id=instance.id).exists():
        instance.__original_object = sender.objects.get(id=instance.id)


@receiver(post_delete, sender=Site)
@receiver(post_delete, sender=DeviceRole)
@receiver(post_delete, sender=DeviceType)
@receiver(post_delete, sender=Prefix)
def save_group_instance(sender, instance, **kwargs):
    rq_queue = get_queue(queue_name)
    for inventory in AWXInventory.objects.all():
        if inventory.enabled == True:
            params = {"inventory": inventory, "sender": sender, "instance": instance}
            rq_queue.enqueue("netbox_awx_plugin.synchronization.delete_group", **params)


@receiver(post_save, sender=Device)
def save_device(sender, instance, created, **kwargs):
    device = sender.objects.get(id=instance.id)
    if not device.primary_ip4 is None and device.primary_ip4.dns_name:
        rq_queue = get_queue(queue_name)
        for inventory in AWXInventory.objects.all():
            if inventory.enabled == True:
                params = {"inventory": inventory, "sender": sender, "instance": device}
                rq_queue.enqueue("netbox_awx_plugin.synchronization.sync_host", **params)


@receiver(post_save, sender=Interface)
def save_interface(sender, instance, **kwargs):
    save_device(Device, instance.device, **kwargs)
